<?php

namespace App\HttpController\Google;

use App\Key\K;
use App\Tools\Email\Log;
use App\Tools\GSession;
use App\Tools\Proxy;
use App\Tools\ProxyManager;
use App\Tools\Shell;
use App\Tools\System;
use App\Tools\WechatNotice;
use EasySwoole\Component\TableManager;
use EasySwoole\Component\Timer;
use EasySwoole\EasySwoole\ServerManager;
use EasySwoole\Http\Message\Status;
use EasySwoole\Http\Message\UploadFile;
use Swlib\SaberGM;
use Swoole\Coroutine;


class Api extends Base
{
    protected function onRequest(?string $action): ?bool
    {
        parent::onRequest($action);
//        if (!current($this->request()->getHeader('bigemap'))&&!$this->request()->getQueryParam('bigemap')){
//            $this->writeJson(500, 'bigemap-server error');
//            return false;
//        }
        $response = $this->response();
        $response->withHeader('Access-Control-Allow-Origin', '*');//允许所有跨域
        $response->withHeader('Access-Control-Allow-Methods', 'GET,POST,PUT,OPTIONS');
        $response->withHeader('Access-Control-Allow-Credentials', 'true');
        $response->withHeader('Access-Control-Allow-Headers', 'Content-Type, token,Authorization, bigemap,X-Requested-With');
        if ($this->request()->getMethod() === 'OPTIONS') {
            $response->withStatus(Status::CODE_OK);
            return false;
        }
        return true;
    }

    public function statistics()
    {
        $data = $this->getRequestData();
        if (!$data[K::KEY_SERVER_PORT]) $data[K::KEY_SERVER_PORT] = 9502;
        $data['ip'] = $this->ip;
        System::serverData($this->ip . '.' . $data[K::KEY_SERVER_PORT], $data);
        $this->jsonSuccess('success');
    }

    public function info()
    {
        foreach (TableManager::getInstance()->get(K::TABLE_FIRST_CACHE) as $v) {
            $v['time'] = date('Y-m-d H:i:s', $v['time']);
            $data['first'][] = $v;
        }
        foreach (TableManager::getInstance()->get(K::TABLE_SECOND_CACHE) as $v) {
            $v['time'] = date('Y-m-d H:i:s', $v['time']);
            $data['second'][] = $v;
        }
        foreach (TableManager::getInstance()->get(K::TABLE_QUERY_LIST) as $k => $v) {
            if ($v['first_time']) $v['first_time'] = date('Y-m-d H:i:s', $v['first_time']);
            if ($v['second_time']) $v['second_time'] = date('Y-m-d H:i:s', $v['second_time']);
            $data['query'][$k] = $v;
        }
        $data['config'] = System::config();
        $this->writeJson(200, $data);
    }

    public function server()
    {
        $data = $this->getRequestData();
        if (!$data['ip']) {
            return $this->jsonError('没有提供地址');
        }
        //如果没有提交端口号，那么我将尝试使用默认的地址
        if (!$data[K::KEY_SERVER_PORT]) $data[K::KEY_SERVER_PORT] = 9502;
        //获取全部的数据
        $serverData = System::serverData();
        $action = $data['action'];
        $mergeData = false;
        unset($data['action']);
        switch ($action) {
            case 'update-server':
            case 'stop-server':
            case 'start-server':
            case 'update-manager':
                if ($data[K::KEY_SERVER_PORT] == 9502) {
                    //如果是主程序的端口，那么交给9504去处理
                    $url = $data['ip'] . ':9504/?port=' . $data[K::KEY_SERVER_PORT] . '&action_name=' . $action;
                    $method = 'GET';
                    $saveData = [];
                } else {
                    //否则交给9502去处理
                    $url = $data['ip'] . ':9502/google/server/index';
                    $method = 'POST';
                    $saveData = $this->getRequestData();
                }
                break;
            case 'create-server':
                $url = $data['ip'] . ':9502/google/server/index';
                $method = 'POST';
                if (!is_numeric($data[K::KEY_SERVER_PORT]) || $data[K::KEY_SERVER_PORT] < 9520) {
                    return $this->jsonError('服务器端口错误!');
                }
                if (!is_numeric($data[K::KEY_PROXY_PORT]) || $data[K::KEY_PROXY_PORT] < 19520) {
                    return $this->jsonError('代理端口错误!');
                }
                $saveData = $this->getRequestData();
                break;
            case 'log-server':
                $url = $data['ip'] . ':9502/google/api/log?port=' . $data[K::KEY_SERVER_PORT];
                $method = 'GET';
                $saveData = [];
                break;
            case 'second-download':
                $url = $data['ip'] . ':9502/google/api/download?day=' . $data['day'] . '&port=' . $data[K::KEY_SERVER_PORT];
                $method = 'GET';
                $saveData = [];
                break;
            case 'get-proxy-server':
                $url = $data['ip'] . ':' . $data[K::KEY_SERVER_PORT] . '/google/api/proxy';
                $method = 'GET';
                $saveData = [];
                break;
            case 'edit-subscribe':
                //订阅地址
                $url = $data['ip'] . ':' . $data[K::KEY_SERVER_PORT] . '/google/api/proxy';
                $method = 'POST';
                $saveData = $data;
                break;
            case 'test-server':
                $url = $data['ip'] . ':' . $data[K::KEY_SERVER_PORT] . '/google/api/test';
                $method = 'GET';
                $saveData = [];
                break;
            case 'select-proxy':
                $mergeData = true;
                $url = $data['ip'] . ':' . $data[K::KEY_SERVER_PORT] . '/google/api/proxy';
//                $url =  'http://localhost:9502/google/api/proxy';
                $saveData = $data;
                $method = 'put';
                break;
            case 'config':
            case 'edit-proxy':
                $mergeData = true;
                $serverId = intval(trim(str_replace(['history'], [''], $data[K::KEY_SERVER_ID])));
                if ($serverId > 1000) {
                    $url = $data['ip'] . ':9502/google/api/config?port=' . $data[K::KEY_SERVER_PORT];
                } else {
                    $url = $data['ip'] . ':' . $data[K::KEY_SERVER_PORT] . '/google/api/config';
                }
                if ($data[K::KEY_PARENT_IP]) {
                    //如果提交了父级IP，那么要判断一次，父级IP的正确性
                    try {
                        SaberGM::get($data[K::KEY_PARENT_IP] . '/google/api/info', ['timeout' => 3]);
                    } catch (\Throwable $exception) {
                        return $this->jsonError($data[K::KEY_PARENT_IP] . '设置不正确!');
                    }
                }
                if ($data[K::KEY_SERVER_ID]) {
                    //判断如果有提交gg18_要判断有没有重名
//                    foreach ($serverData as $k=>$v){
//                        if ($v[K::KEY_SERVER_ID]==$data[K::KEY_SERVER_ID]&&$k!=$data['ip']&&$v[K::KEY_SERVER_DEADLINE]>time()+86400*2){
//                            //同时还需要是服务器的过期时间大于2天的才会认为有重名
//                            return $this->jsonError($data[K::KEY_SERVER_ID].' 此名称已经被占用!');
//                        }
//                    }
                }
                $saveData = $data;
                $method = 'put';
                break;
            case 'delete-server':
                if (time() - $serverData[$data['ip']]['time'] < 60 * 60 * 2) {
                    return $this->jsonError('此服务器在最近2小时之内有提交');
                }
                $config = System::config(K::KEY_CACHE_PATH);
                unlink($config . '/ip/' . $data['ip'] . '.' . $data[K::KEY_SERVER_PORT] . '.ip');
                return $this->jsonSuccess('success');
                break;
            default:
                return $this->jsonError('没有此方法--' . $action);
        }

        try {
            switch (strtoupper($method)) {
                case 'GET':
                    $result = SaberGM::get($url, ['timeout' => 7, 'headers' => ['bigemap' => 1]]);
                    break;
                case 'POST':
                    $result = SaberGM::post($url, $saveData, ['timeout' => 7, 'headers' => ['bigemap' => 1]]);
                    break;
                case 'PUT':
                    $result = SaberGM::put($url, $saveData, ['timeout' => 7, 'headers' => ['bigemap' => 1]]);
                    break;
            }
            $resultData = $result->getBody()->getContents();
            if (json_decode($resultData, true)) {
                $resultData = json_decode($resultData, true);
                if ($resultData['data'] && $mergeData) {
                    $resultData['data'] = array_merge($serverData[$data['ip']], $resultData['data']);
                    //同时马上更新本地的数据，方便同步
                    System::serverData($data['ip'] . '.' . $data[K::KEY_SERVER_PORT], $resultData['data']);
                }
                $this->response()->write(json_encode($resultData, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
            } else {
                $this->response()->write($resultData);
            }
        } catch (\Exception $exception) {
            $this->jsonError($exception->getMessage());
        }
    }

    public function subscribe()
    {
        $manager = ProxyManager::getInstance();
        $serverData = $manager->getAllServer();
        if (!$serverData) {
            $serverData = ['subscribe' => []];
        } else {
            $serverData = ['subscribe' => $serverData];
        }
        switch ($this->request()->getMethod()) {
            case 'GET':
                foreach ($serverData['subscribe'] as $k => $v) {
                    if ($v['in_use'] >= $v['limit'] && !$this->request()->getQueryParam('all')) {
                        unset($serverData['subscribe'][$k]);
                    }
                }
                $this->jsonSuccess($serverData);
                break;
            case 'POST':
                //这里是定期检查流星的使用情况然后更新数据的
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('NO id');
                $manager->save('subscribe', ['id' => $data['id']], ['left' => $data['left'], 'left_time' => $data['left_time']]);
                return $this->jsonSuccess('ok');
                break;
        }
    }
    public function notice(){
        $handler=new Handler();
        $type=$this->request()->getQueryParam('type');
        switch ($type){
            case '1':
                $handler->noticeWechat();
                break;
            case '2':
                //这里主要是进行服务的更新操作，比如服务器流量异常
                $serverId=$this->request()->getQueryParam('id');
                if (!$serverId) return $this->jsonError('no serverid');
                $allServer=System::serverData();
                foreach ($allServer as $v){
                    if (System::getServerIdNumber($v[K::KEY_SERVER_ID])==$serverId){
                        try {
                            SaberGM::post(K::MANAGE_SERVER_ADDR_9502.'/google/api/server?bigemap=1',['action'=>'update-server','ip'=>$v['ip'],K::KEY_SERVER_PORT=>$v[K::KEY_SERVER_PORT]],['timeout'=>5]);
                        }catch (\Throwable $throwable){
                            return  $this->jsonError($throwable->getMessage());
                        }
                        break;
                    }
                }
                return  $this->jsonSuccess('ok');
                break;
        }
    }
    public function proxy()
    {
        $cachePath = System::config(K::KEY_CACHE_PATH);
        switch ($this->request()->getMethod()) {
            case 'GET':
                //获取当前的数据
                $data = System::config();
                $subscribe = Proxy::getAllProxy(true);
                $this->jsonSuccess(['data' => $data, 'subscribe' => $subscribe]);
                break;
            case 'POST':
                //这里会是服务器自动启动代理之后，会提交上来当前使用的代理信息，如果有重复的，那么要告诉服务器，尝试更换代理
                $data = $this->getRequestData();
                $manager = ProxyManager::getInstance();
                switch ($data['code']) {
                    case  '200':
                        $useServer = $data['result'];
//                        if (in_array(trim($useServer[K::KEY_SERVER_ID]),['history1000','history1001','history1031','history1052'])) return $this->jsonSuccess(['action'=>'ok']);
                        //首先根据根据KEY来更新此代理的IP
                        if ($useServer[K::KEY_PROXY_CURRENT][1] && $useServer[K::KEY_PROXY_HISTORY_ENABLE] && $useServer[K::KEY_GOOGLE_HISTORY_ENABLE]) {
                            $usedKey = $manager->select('server_list', ['key' => $useServer[K::KEY_PROXY_CURRENT][1]]);
                            foreach ($usedKey as $v) {
                                if ($useServer[K::KEY_PROXY_IP] && $v['ip'] != $useServer[K::KEY_PROXY_IP]) {
                                    $manager->save('server_list', ['id' => $v['id']], ['ip' => $useServer[K::KEY_PROXY_IP]]);
                                }
                            }
                        }
                        //然后IP来判断
                        if ($useServer[K::KEY_PROXY_IP] && $useServer[K::KEY_PROXY_HISTORY_ENABLE] && $useServer[K::KEY_GOOGLE_HISTORY_ENABLE]) {
                            $usedIp = $manager->select('server_list', ['ip' => $useServer[K::KEY_PROXY_IP]]);
                            foreach ($usedIp as $v) {
                                if ($v['server_id'] && $v['server_id'] != $useServer[K::KEY_SERVER_ID] && time() - $v['update_time'] < 60 * 3) {
                                    return $this->jsonSuccess(['action' => 'used', 'used' => $v]);
                                }
                            }
                            //如果上面当前这个IP的所有订阅都没有使用 ，那么就保存信息
                            foreach ($usedIp as $v) {
                                $manager->save('server_list', ['id' => $v['id']], ['status' => 2, 'update_time' => time(), 'server_id' => $useServer[K::KEY_SERVER_ID]]);
                            }
                        }
                        //然后再选择出所有和当前的服务器ID相关的代理
                        if ($useServer[K::KEY_SERVER_ID]) {
                            $usedId = $manager->select('server_list', ['server_id' => $useServer[K::KEY_SERVER_ID]]);
                            foreach ($usedId as $v) {
                                if (!in_array($v['status'], [1, 2])) continue;
                                if ($useServer[K::KEY_PROXY_HISTORY_ENABLE]) {
                                    if ($useServer[K::KEY_PROXY_IP] && $v['ip'] != $useServer[K::KEY_PROXY_IP]) {
                                        //只要IP不相同，那么认为此服务器已经没有使用这个代理了
                                        $manager->save('server_list', ['id' => $v['id']], ['status' => 1, 'update_time' => time(), 'server_id' => '']);
                                    }
                                } else {
                                    $manager->save('server_list', ['id' => $v['id']], ['status' => 1, 'update_time' => time(), 'server_id' => '']);
                                }
                            }
                        }
                        return $this->jsonSuccess(['action' => 'ok']);
                        break;
                    case '403':
                        //这里是在启动代码的时候，如果启动成功了，但是获取是403错误，那么要保存一次
                        $noticeData = ['action' => 403, 'content' => []];
                        foreach ($data['result'] as $v) {
                            $info = json_decode($v['info'], true);
                            $noticeData['content'][] = $info['ps'] ? $info['ps'] : $v['key'];
                            $manager->save('server_list', ['key' => $v['key']], ['status' => 100, 'server_id' => '', 'next_check_time' => time() + 3600 * 10 + 3600 * 2 * mt_rand(1, 100) / 100]);
                        }
                        if ($noticeData['content']) {
                            array_unshift($noticeData['content'], '启动代理，检查被封' . count($noticeData['content']) . '台');
                            $noticeData['content'] = implode("\r\n", $noticeData['content']);
                            $noticeData['key'] = md5($noticeData['content']);
                            WechatNotice::notice($noticeData);
                        }
                        break;
                    case '403_key':
                        //这里是在自动获取session的时候，获取是403错误，那么要保存一次
                        if (!$data['result']) {
                            System::log('403_key no key');
                            return $this->jsonError('no key');
                        }
                        $serverData = $manager->find('server_list', ['key' => $data['result']]);
                        $serverData['info'] = json_decode($serverData['info'], true);
                        $updateData = ['status' => 100, 'server_id' => '', 'forbidden_time' => time(), 'next_check_time' => time() + 3600 * 10 + 3600 * 2 * mt_rand(1, 100) / 100];
                        if ($serverData['server_id']) $updateData['forbidden_server_id'] = $serverData['server_id'];
                        $manager->save('server_list', ['key' => $data['result']], $updateData);
                        if ($serverData['ip']) {
                            //如果有IP，以IP来更新数据库,如果直接使用IP来更新，这里要注意一下，一定要判断有没有IP信息，因为有可能当前的服务器，还没有写入IP，那么有会将所有的没有IP的服务器，更新为 403!!
                            $manager->save('server_list', ['ip' => $serverData['ip']], $updateData);
                        }
                        $noticeData = ['action' => 403, 'content' => '服务器运行中，获取 Session 被封 1 台' . "\r\n"];
                        $noticeData['content'] .= $serverData['info']['ps'] ? $serverData['info']['ps'] : '未知名称';
                        $noticeData['key'] = $data['result'];
                        WechatNotice::notice($noticeData);
                        return $this->jsonSuccess($noticeData);
                        break;
                }
                break;
            case 'PUT':
                $data = $this->getRequestData();
                $serverData = Proxy::getAllProxy(true);
                $subscribeId = array_column($serverData, 'id');
                if (!in_array($data[K::KEY_PROXY_CURRENT][0], $subscribeId)) {
                    return $this->jsonError('选择代理组不正确!');
                }
                $selected = '';
                foreach ($serverData as $vv) {
                    if ($vv['id'] == $data[K::KEY_PROXY_CURRENT][0]) {
                        foreach ($vv['children'] as $v) {
                            if ($v['key'] == $data[K::KEY_PROXY_CURRENT][1]) {
                                if ($v['status'] != '1' || ($v['status'] == '2' && time() - $v['update_time'] < 60 * 10)) {
                                    return $this->jsonError('此代理不可以使用--' . $v['status']);
                                }
                                $selected = $v;
                                break;
                            }
                        }
                    }
                }
                if (!$selected) return $this->jsonError('选择代理不正确!');
                $selectedConfig = json_decode($selected['info'], true);
                if (!$selectedConfig) return $this->jsonError('解析代理出错!');
                if ($data[K::KEY_PROXY_HISTORY_ENABLE]) {
                    //如果要启动代理 ，那么，我们需要测试一下代理
                    $v2rayPath = $cachePath . '/v2ray/v2ray';
                    if (!file_exists($v2rayPath)) {
                        $status = Shell::getZipAndExtract('http://google.zhangwei.fun/xt/v2ray.zip', $cachePath . '/v2ray');
                        if (!$status) return $this->jsonError('解压v2ray文件夹失败');
                    }
                    $configTemplate = Proxy::getBaseTemplate();
                    if (!$configTemplate) return $this->jsonError('获取模版失败!');
                    $testPort = 50005;
                    if (!Proxy::startProxy($configTemplate, $selectedConfig, $testPort)) {
                        return $this->jsonError(Proxy::getError());
                    }
                    Coroutine::sleep(2);
                    //启动成功了，那么进行一次测试，先测试谷歌，证明是通的
                    try {
                        SaberGM::get('https://www.google.com', ['timeout' => 3, 'proxy' => 'socks5://127.0.0.1:' . $testPort]);
                    } catch (\Throwable $exception) {
                        return $this->jsonError('test google  ' . $exception->getCode());
                    }
                    try {
                        $defaultData = GSession::getDefaultQueryData('second');
                        $sessionData = SaberGM::post($defaultData['url'], $defaultData['data'], ['headers' => $defaultData['headers'], 'proxy' => 'socks5://127.0.0.1:' . $testPort]);
                        $sessionData = $sessionData->getBody()->getContents();
                        //保存起来，方便使用
                        GSession::saveSession($sessionData);
                        System::log($sessionData, 'test session');
                    } catch (\Throwable $exception) {
                        if ($exception->getCode() == 403) {
                            //如果是403错误，那么要通知主服务器
                            Proxy::noticeForbiddenServer($selected['key']);
                        }
                        return $this->jsonError($exception->getMessage());
                    }
                    //清空代理的IP
                    System::config(K::KEY_PROXY_IP, '');
                    $proxyPort = System::config(K::KEY_PROXY_PORT);
                    if (!$proxyPort) $proxyPort = 50010;
                    //这里调用成功了，那么就启用谷歌代理
                    if (!Proxy::startProxy($configTemplate, $selectedConfig, $proxyPort)) {
                        return $this->jsonError(Proxy::getError());
                    }
                    Timer::getInstance()->after(5000, function () use ($testPort) {
                        //停止测试的代理
                        echo 'stop shell', PHP_EOL;
                        Shell::shellKill('v2ray/config' . $testPort);
                    });
                    $data[K::KEY_PROXY_HISTORY] = 'socks5://127.0.0.1:' . $proxyPort;
                    $ip = Proxy::getProxyIp('socks5://127.0.0.1:' . $testPort);
                    System::log($ip, 'proxy ip');
                    if ($ip) System::config(K::KEY_PROXY_IP, $ip);
                }
                foreach ($data as $k => $v) {
                    System::config($k, $v);
                }
                //进入到了这里，那么认为代理已经通过了验证，直接启用服务器
//                System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '1');
                return $this->jsonSuccess(System::config());
                break;
        }
    }

    public function sub()
    {
        switch ($this->request()->getMethod()) {
            case 'GET':
                $subServer = Proxy::subServer();
                return $this->jsonSuccess($subServer);
                break;
            case 'POST':
                //用于子服务器的提交数据
                $data = $this->getRequestData();
                if (!$data['port']) return $this->jsonError('no port');
                $config = System::config();
                $currentIp = $config[K::KEY_CURRENT_IP];
                $serverId = str_replace(['history'], [''], $config[K::KEY_SERVER_ID]);
                $serverId = intval(trim($serverId));
                if ($serverId >= 1000 || $this->ip == $currentIp) {
                    $host = '127.0.0.1';
                } else {
                    $host = $this->ip;
                }
//                $host=$this->ip==$currentIp?'127.0.0.1':$this->ip;
                Proxy::subServer(['host' => $host, 'port' => $data['port']]);
                return $this->jsonSuccess('success');
                break;
            case 'DELETE':
                //如果子服务器在运行的过程中，出现了停用的情况，那么反向父级服务器提交信息，删除这个子服务器
                $port=$this->request()->getQueryParam('port');
                Proxy::removeSubServer($port);
                break;
        }
    }

    public function log()
    {
        $port = $this->request()->getQueryParam('port');
        if ($port == 9502) {
            $logPath = EASYSWOOLE_ROOT . '/Log/swoole.log';
        } else {
            $logPath = dirname(EASYSWOOLE_ROOT) . '/easyswoole' . $port . '/Log/swoole.log';
        }
        $shell = 'tail -n 1000 ' . $logPath;
        exec($shell, $log, $status);
        $log = array_map('base64_encode', $log);
        if (!$log) $log = [];
        $fileExists = file_exists($logPath) ? 1 : 0;
        $this->response()->write(json_encode(['code' => 200, 'log' => $log, 'execStatus' => $status, 'shell' => $shell, 'file' => $fileExists]));
    }

    public function config()
    {
        switch ($this->request()->getMethod()) {
            case 'GET':
                return $this->jsonSuccess(System::getServerInfo());
                break;
            case 'PUT':
                $port = $this->request()->getQueryParam('port');
                $data = $this->getRequestData();
                if ($port && $port != 9502) {
                    //这个是内网才会进入到这里
                    $data = $this->getRequestData();
                    System::log($data, 'received data');
                    try {
                        $data = SaberGM::put('http://127.0.0.1:' . $port . '/google/api/config?bigemap=1', $data, ['timeout' => 2]);
                        $data = $data->getBody()->getContents();
                        $data = json_decode($data, true);
                        $this->jsonSuccess($data['data']);
                    } catch (\Throwable $exception) {
                        System::log($exception->getMessage(), 'inner network failed');
                        return $this->jsonError($exception->getMessage());
                    }
                } else {
                    System::log($data, 'received data');
                    foreach ($data as $k => $v) {
                        if (!is_array($v)) System::config($k, $v);
                    }
                    $this->jsonSuccess(System::getServerInfo());
                }
                break;
        }
    }

    public function download()
    {
        $day = $this->request()->getQueryParam('day');
        $port = $this->request()->getQueryParam('port');
        $cachePath = System::config(K::KEY_CACHE_PATH);
        if ($port != '9502') {
            $cachePath = dirname($cachePath) . '/' . $port;
        }
        switch ($day) {
            case '1-0':
            case '1-1':
            case '1-2':
            case '1-3':
            case '2-0':
            case '2-1':
            case '2-2':
            case '2-3':
                $deltaDay = explode('-', $day);
                $fileTime = time() - intval($deltaDay[1]) * 86400;
                $filePath = $cachePath . '/post' . $deltaDay['0'] . '-' . date('Y-m-d', $fileTime);
                $origin = explode(PHP_EOL, file_get_contents($filePath));
                $origin = array_unique($origin);
                foreach ($origin as $v) {
                    $v = json_decode($v, true);
                    if (!$v) continue;
                    $data[] = $v;
                }
                break;
            case 'session':
                $filePath = $cachePath . '/session-ok';
                $data = explode(PHP_EOL, file_get_contents($filePath));
                $data = array_filter(array_values(array_unique($data)));
                break;
            case 'day':
                $filePath = $cachePath . '/day';
                $origin = array_filter(explode(PHP_EOL, file_get_contents($filePath)));
                foreach ($origin as $v) {
                    $v = json_decode($v, true);
                    if (!$v) continue;
                    $data[] = $v;
                }
                break;
        }
        if (empty($data)) $data = [];
        $this->jsonSuccess(['log' => $data]);
    }

    public function test()
    {
        $token = GSession::test();
        if ($token['code'] == 200) {
            GSession::saveSession($token['token']);
            $token['token'] = base64_encode($token['token']);
            $this->response()->write(json_encode($token));
        } else {
            $this->response()->write(json_encode($token));
        }
    }

    public function testa()
    {
        $currentIndex = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG)->get(K::KEY_SLEEP_INDEX, 'number');
        $this->response()->write($currentIndex);
    }

    public function upload(){
        switch ($this->request()->getMethod()){
            case 'POST':
                /**
                 * @var $file UploadFile
                 */
                $file=$this->request()->getUploadedFile('file');
                if (!$file) return $this->jsonError('file not found',4801);
                $fileName=$file->getClientFilename();
                $extension=strtolower(pathinfo($fileName,PATHINFO_EXTENSION));
                $path= dirname(EASYSWOOLE_ROOT).'/uploads/'.date('Ymd');
                $savePath=$path.'/'.ceil(microtime(true)*10000).'.'.$extension;
                try {
                    $file->moveTo($savePath);
                }catch (\Throwable $exception){
                    echo $exception->getMessage(),PHP_EOL;
                    return $this->jsonError('save error',4809);
                }
                return  $this->jsonSuccess(['src'=>$savePath,'title'=>$fileName]);
                break;
        }
    }
}
