<?php

namespace App\HttpController\Google;

use EasySwoole\EasySwoole\ServerManager;

class Base extends \App\HttpController\Base\Base
{
    public $ip = '';

    protected function onRequest(?string $action): ?bool
    {
        parent::onRequest($action);
        $headerIp = $this->request()->getHeader('bigemapip');
        $headerIp = current($headerIp);
        if (filter_var($headerIp, FILTER_VALIDATE_IP)) {
            //如果是传递过来了IP，那么使用传递过来的IP
            $this->ip = $headerIp;
        } else {
            $ip = ServerManager::getInstance()->getSwooleServer()->getClientInfo($this->request()->getSwooleRequest()->fd);
            $this->ip = $ip['remote_ip'];
        }
        return true;
    }

    protected function getCookie()
    {
        $cookie = $this->request()->getCookieParams();
        return array_map(function ($v) {
            return str_replace([' '], ['+'], $v);
        }, $cookie);
    }

    protected function setStatusCode($code)
    {
        $this->response()->withStatus($code);
        $this->response()->getSwooleResponse()->setStatusCode($code);
    }

    protected function header($key = '', $value = '')
    {
        if ($key && $value) {
            $this->response()->getSwooleResponse()->header($key, $value);
            $this->response()->withAddedHeader($key, $value);
            return true;
        }
        $header = [];
        $request = $this->request()->getSwooleRequest();
        foreach ($request->header as $k => $v) {
            //所有包含bigemap的都是自定义header，都不要
            if (strpos($k, 'bigemap') !== false) continue;
            switch (strtolower($k)) {
                case 'host':
                    //$header[$k] = "kh.google.com";
                    break;
                case 'connection':
                    //$header[$k] = 'Keep-Alive';
                    break;
                //这个是平台的一个标治信息，
                case 'cf':
                    break;
                default:
                    $header[$k] = $v;
            }
        }
        return $header;
    }

    protected function gc()
    {
        $this->ip = '';
        parent::gc(); // TODO: Change the autogenerated stub
    }
}