<?php

namespace App\HttpController\History;

use App\HttpController\Base\Base;
use App\Key\K;
use App\Model\History\ServerList;
use App\Model\History\SubscribeList;
use App\Tools\Proxy;
use EasySwoole\ORM\DbManager;
use Swoole\Coroutine;

class Subscribe extends Base
{
    //前端页面展示所有的订阅

    public function index()
    {
        $model = new SubscribeList();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $data = $model->getPageData();
                foreach ($data['data'] as $k => $v) {
                    $data['data'][$k]=$model->getReadAbleData($v['id']);
                }
                return $this->jsonSuccess($data);
                break;
            case 'POST':
                $data = $this->getRequestData();
                DbManager::getInstance()->startTransaction(K::DB_CONNECT_HISTORY);
                $result = $model->groupAdd('add', $data);
                if (!$result) return $this->jsonError($model->getError());
                $parse = Proxy::parseUrl($data['url'], $data['url_file'][0]['url']);
                if ($parse) {
                    $this->reSubscribe($parse, $result);
                } else {
                    //如果有地址，或者是上传了文件，但没有数据，那么认为解析有问题
                    if ($data['url'] || $data['url_file'][0]['url']) {
                        return $this->jsonError('解析失败');
                    }
                }
                DbManager::getInstance()->commit(K::DB_CONNECT_HISTORY);
                $result=$model->getReadAbleData($result['id']);
                return $this->jsonSuccess($result);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                DbManager::getInstance()->startTransaction(K::DB_CONNECT_HISTORY);
                $result = $model->groupSave($data['id'], 'edit', $data);
                if (!$result) return $model->getError();
                if ($data['resubscribe'] || $data['url_file'][0]['url']) {
                    //需要重新订阅
                    $server = Proxy::parseUrl($data['url'], $data['url_file'][0]['url']);
                    if (!$server) {
                        return $this->jsonError('没有数据');
                    }
                    $sum = count($server);
                    $this->reSubscribe($server, $result);
                    $result['_sum_'] = $sum;
                } else {
                    $result['_sum_'] = 0;
                }
                DbManager::getInstance()->commit(K::DB_CONNECT_HISTORY);
                $result=$model->getReadAbleData($result['id']);
                return $this->jsonSuccess($result);
                break;
            case 'DELETE':
                $data = $this->getRequestData();
                $subscribe = $model->where(['id' => $data['id']])->get();
                if (!$subscribe) return $this->jsonError('没有此条数据');
                $itemModel = new ServerList();
                $itemModel->where(['pid' => $data['id']])->destroy();
                $subscribe->destroy();
                return $this->jsonSuccess($subscribe->toArray());
                break;
        }
    }

    private function reSubscribe($server, $subscribe)
    {
        $modelServer = new ServerList();
        $old = $modelServer::getAllServer($subscribe['id'], 'key');
        //判断哪些是旧的数据，直接删除掉
        $removeItems = array_diff_key($old, $server);
        foreach ($removeItems as $v) {
            $modelServer->where(['id' => $v['id']])->destroy();
        }
        //增加了的代理服务器，那么要添加上
        $newItems = array_diff_key($server, $old);
        foreach ($newItems as $v) {
            $insert = [
                'info' => $v,
                'base_type' => $subscribe['base_type'],
                'title' => $v['name'],
                'pid' => $subscribe['id'],
                'key' => $v['server'] . $v['port'] . $v['uuid'],
            ];
            $obj = $modelServer::create($insert);
            $obj->save();
        }
        //交集的部分就合并更新
        $common = array_intersect_key($old, $server);
        foreach ($common as $k => $v) {
            $update = [
                'info' => $server[$k],
                'title' => $server[$k]['name'],
            ];
            $modelServer->where(['id' => $v['id']])->update($update);
        }
    }
    //测试这个代理是不是连通的，主要是前端页面使用
    public function test()
    {

        $model = new ServerList();
        $id = $this->request()->getQueryParam('id');
        if (!$id) return $this->jsonError('提交出错!');
        $data = $model->get($id);
        if (!$data) return $this->jsonError('没有此条数据!');
        $data = $data->toArray();
        $template = Proxy::getBaseTemplate();
        if (!$template) return $this->jsonError('获取模版出错!');
        $port = K::TEST_PROXY_PORT;
        $result = Proxy::startProxy($template, $data['info'], $port);
        if (!$result) return $this->jsonError(Proxy::$error);
        Coroutine::sleep(4);
        $ip = Proxy::getProxyIp('socks5://127.0.0.1:' . $port);
        if (!$ip) {
            return $this->jsonError('测试失败');
        }
        $result = $model->groupSave($id, 'edit', ['ip' => $ip]);
        if (!$result) return $this->jsonError($model->getError());
        $this->jsonSuccess(['ip' => $ip, 'template' => $template, 'result' => $result]);
    }

    //这个是前端管理页面使用的接口
    public function server()
    {
        $model = new ServerList();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $data = $model->getPageData();
                $all = SubscribeList::getAll();
                //增加这个mapping是方便前端来展示数据 ,key=>value的属性
                $data['mapping'] = [];
                foreach ($all as $v) {
                    $data['mapping'][] = ['label' => $v['title'], 'value' => $v['id']];
                }
                return $this->jsonSuccess($data);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                $result = $model->groupSave($data['id'], 'edit', $data);
                if (!$result) return $model->getError();
                return $this->jsonSuccess($result);
                break;
        }
    }

    //所有的服务器拉取可以使用的订阅，从而开启订阅
    public function proxy()
    {
        $model = new SubscribeList();
        $serverModel = new ServerList();
        $all = $model::getAll();
        $need = [];
        foreach ($all as $v) {
            if (!$v['enable']) continue;
            $children = $serverModel::getServerUsage($v['id']);
            $children['servers'] = array_filter($children['servers'], function ($v) {
                return $v['status'] == K::PROXY_SERVER_OK || ($v['status'] == K::PROXY_SERVER_IN_USE && (time() - $v['update_time']) > 60 * 10);
            });
            if (!$children['servers'] || $children['use'] >= $v['limit']) continue;
            $v['servers'] = $children['servers'];
            $need[] = $v;
        }
        return $this->jsonSuccess($need);
    }
}
