<?php

namespace App\Model\History;

class ServerInfo extends Base
{
    protected $updateTime = 'update_time';
    protected $autoTimeStamp = true;
    protected $pk = 'server_id';
    protected $createTime = 'create_time';
    protected $tableName = 'server_info';
    protected $tableField = [
        'info' => [],
        'update_time' => [],
        'create_time' => [],
    ];

    protected $operation = [
        'add' => [
            'field' => '*',
        ],
        'edit' => [
            'field' => '*'
        ],
        'search' => [
            'field' => ['pid', 'server_id', 'ip', 'status', 'forbidden_server_id', 'title']
        ]
    ];

    protected $validate = [
        'title' => 'require',
    ];

    protected $validateMessage = [
        'title.require' => '标题不可以为空',
    ];

    protected function getInfoAttr($info)
    {
        return $this->jsonToArray($info);
    }

    protected function setInfoAttr($info)
    {
        return $this->arrayToJson($info);
    }

    public static function getServerUsage($pid)
    {
        $all = static::getAllServer($pid);
        $usage = ['ip' => [], 'use' => [], 'unused' => [], 'forbidden' => [], 'total' => count($all)];
        foreach ($all as $v) {
            $usage['ip'][] = $v['ip'];
            if ($v['server_id'] && time() - $v['update_time'] < 60 * 10) $usage['use'][]=$v['server_id'];
            if (!$v['update_time']) $usage['unused'][]=$v['key'];
            if ($v['status'] == 2) $usage['forbidden'][]=$v['key'];
        }
        foreach (['use','unused','forbidden','ip'] as $v){
            $usage[$v]=array_unique($usage[$v]);
            $usage[$v]=count($usage[$v]);
        }
        $usage['servers'] = $all;
        return $usage;
    }

    public static function getAllServer($pid = '', $key = '')
    {
        if ($pid) {
            $data = static::create()->where(['pid' => $pid])->all();
        } else {
            $data = static::create()->all();
        }
        $data = array_map(function ($v) {
            return $v->toArray();
        }, $data);
        if (!$key) {
            return $data;
        }
        $result = [];
        foreach ($data as $v) {
            $result[$v[$key]] = $v;
        }
        return $result;
    }

}
